; TSM V1.5
; Compiles with DASM and is tested with Z26. 
;
; Done by Manuel Polik (cybergoth@nexgo.de)
; (GFX kernel by Piero Cavina)
;
    processor 6502
    include vcs.h
    include macro.h
    include music.h

NTSC                    = 0
PAL                     = 1 

    SEG.U   variables
    ORG     $80

;;;;;;;;;; TSM data ;;;;;;;;;;
frameCounter     ds 1
tempoCount       ds 1
soundPointer     ds 2
;;;;;;;;;; TSM data ;;;;;;;;;;

    SEG     Bank0
    ORG     $F000

;;;;;;;;;; TSM init ;;;;;;;;;;
TSMInit
            LDA #<channeldata
            STA soundPointer
            LDA #>channeldata
            STA soundPointer+1
            RTS
;;;;;;;;;; TSM init ;;;;;;;;;;

XmasScreen          
                STA WSYNC
                LDX #7
LPP:   
                DEX
                BPL LPP
                STA RESM1
                STA RESM0
                STA HMCLR
                STA WSYNC           ; Finish current line
                STA VBLANK          ; Stop vertical blank
                LDA #$FF
                STA ENAM1
                STA ENAM0
                LDX #190
LK:    
                TXA
                ADC frameCounter
                ASL
                ASL
                STA HMM1
                EOR #$FF
                STA HMM0
                STA COLUP1
                STA COLUP0
                STA WSYNC
                STA HMOVE
                DEX
                BNE LK
                STX ENAM1
                STX ENAM0

            IF COMPILE_VERSION = PAL
                LDX #$36
            ELSE
                LDX #$1E
            ENDIF

Waste           STA WSYNC           ; Waste a line
                DEX                 ;
                BNE Waste           ; if X is not zero, do more lines
                RTS

Start               
        CLEAN_START
        JSR TSMInit

MainLoop
        LDY #$02
        LDA #$00
        STY WSYNC           
        STY VSYNC           
        STY WSYNC           
        STY WSYNC           
        STY WSYNC           
        STA VSYNC           
        JSR VerticalBlank   
        JSR XmasScreen
        JMP MainLoop

VerticalBlank       
            STA WSYNC           ; Finish current line
                STA VSYNC           ; Stop vertical sync
                LDA #$02            ;
                STA VBLANK          ; Start vertical blank

            IF COMPILE_VERSION = PAL
                LDA #$49
            ELSE
                LDA #$2B
            ENDIF
                STA TIM64T          ; Init timer

;;;;;;;;;; TSM code ;;;;;;;;;;

            INC tempoCount
            LDA tempoCount
            EOR #TEMPODELAY
            BNE MusicDone
            STA tempoCount

            LDX #$01
SecondChannel
            TXA
            TAY
            LDA (soundPointer),Y
            BEQ Silencium
            STA AUDF0,X
            STA frameCounter
            LSR
            LSR
            LSR
            LSR
            LSR
            TAY
            LDA soundTypeArray,Y
            STA AUDC0,X

            LDA #15
Silencium
            STA AUDV0,X
            INY
            DEX
            BPL SecondChannel
            INC soundPointer
            INC soundPointer
            BNE IncrementDone
            INC soundPointer+1
IncrementDone
            LDA soundPointer+1
            CMP #>DataEnd
            BNE MusicDone
            LDA soundPointer
            CMP #<DataEnd
            BNE MusicDone
            JSR TSMInit
MusicDone
;;;;;;;;;; TSM code ;;;;;;;;;;


WaitIntimReady      
        LDA INTIM           ; Wait until vertical blank finished
                BNE WaitIntimReady  ;
                RTS

soundTypeArray
    byte 4,6,7,8,15,12,1,3

    include tsmsong.h
DataEnd    

; Reset Vector
        ORG        $FFFC
        .word Start
        .word Start
